VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "RankGrid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ====== Module for manipulating that grid ============
Option Explicit

Private lines_ As Collection
Private page_ As Visio.Page
Private rankHeight_ As Double

Public Function Init(aPage As Visio.Page)
  Set page_ = aPage
  Call ScanShapes
End Function

Public Property Get Size() As Integer
  Size = lines_.Count
End Property

Public Property Get Width() As Double
  If lines_.Count = 0 Then _
    Exit Function
  Width = Abs(lines_(1).CellsU("BeginX") - lines_(1).CellsU("EndX"))
End Property

Public Function GetRankFor(target As Visio.Shape) As Integer
  GetRankFor = RANK_UNREACHABLE
  If lines_.Count = 0 Then _
    Exit Function
    
  Dim yPos As Double: yPos = target.CellsU("PinY")
  If yPos < lines_.Item(lines_.Count).CellsU("PinY") Then _
    Exit Function
  
  Dim nGridShape%: nGridShape = 1
  Dim iShape As Visio.Shape
  For Each iShape In lines_
    If yPos > iShape.CellsU("PinY") Then
      GetRankFor = nGridShape - 1
      Exit Function
    End If
    nGridShape = nGridShape + 1
  Next iShape
End Function

Public Function CreateGrid(nCount%, dHeight As Double, dWidth As Double, dTopMargin As Double, dLeftMargin As Double)
  Call DeleteGrid
  If nCount < 1 Then _
    Exit Function
  
  Dim dOriginX As Double: dOriginX = page_.PageSheet.CellsU("XGridOrigin") + dLeftMargin
  Dim dEndX As Double: dEndX = dOriginX + dWidth
  Dim dOriginY As Double: dOriginY = page_.PageSheet.CellsU("YGridOrigin") - dTopMargin
  rankHeight_ = dHeight
  
  Dim nRank%
  For nRank = 1 To nCount Step 1
    Dim iLine As Visio.Shape: Set iLine = page_.Drop(FindMaster(page_.Document, MASTER_LEVEL), dOriginX, dOriginY)
    Call lines_.Add(iLine)
    With iLine
      .CellsU("BeginX") = dOriginX
      .CellsU("EndX") = dEndX
      .CellsU("BeginY") = dOriginY - (nRank - 1) * rankHeight_
      .CellsU("EndY") = .CellsU("BeginY")
      .Name = PREFIX_GRID + CStr(nRank)
    End With
  Next nRank
End Function

Public Function DeleteGrid()
  Dim iShape As Visio.Shape
  For Each iShape In lines_
    Call iShape.Delete
  Next iShape
  Set lines_ = New Collection
End Function

Public Function MoveUp(target As Visio.Shape) As Boolean
  MoveUp = False
  
  Dim nRank%: nRank = GetRankFor(target)
  If nRank = RANK_UNREACHABLE Then _
    nRank = Size
    
  MoveUp = MoveToRank(target, nRank - 1)
End Function

Public Function MoveDown(target As Visio.Shape) As Boolean
  MoveDown = False
  
  Dim nRank%: nRank = GetRankFor(target)
  If nRank = RANK_UNREACHABLE Then _
    Exit Function
    
  MoveDown = MoveToRank(target, nRank + 1)
End Function

Public Function MoveToRank(target As Visio.Shape, nRank%) As Boolean
  MoveToRank = False
  If nRank > RANK_MAX Or nRank < 0 Then _
    Exit Function
  If Size() = 0 Then _
    Exit Function
  MoveToRank = True
  
  If nRank = 0 Then
    target.CellsU("PinY") = lines_.Item(1).CellsU("PinY") + V_GAP + target.CellsU("Height") / 2#
    Exit Function
  End If
  
  Do While nRank + 1 > lines_.Count
    Dim newShape As Visio.Shape: Set newShape = CreateShape(MASTER_LEVEL, page_)
    Dim prevLine As Visio.Shape: Set prevLine = lines_.Item(lines_.Count)
    Call lines_.Add(newShape)
    With newShape
      .CellsU("BeginX") = prevLine.CellsU("BeginX")
      .CellsU("EndX") = prevLine.CellsU("EndX")
      .CellsU("BeginY") = prevLine.CellsU("BeginY") - rankHeight_
      .CellsU("EndY") = prevLine.CellsU("EndY") - rankHeight_
      .Name = GetNextLineName
    End With
  Loop
  
  target.CellsU("PinY") = (lines_.Item(nRank + 1).CellsU("PinY") + lines_.Item(nRank).CellsU("PinY")) / 2#
End Function

Public Function IncrementWidth(dIncrement As Double) As Boolean
  IncrementWidth = False
  
  If lines_.Count = 0 Then _
    Exit Function
  If dIncrement = 0 Then
    IncrementWidth = True
    Exit Function
  End If
  
  Dim bFirst As Boolean: bFirst = True
  Dim dBeginX As Double
  Dim dWidth As Double
  Dim iLine As Visio.Shape
  For Each iLine In lines_
    If bFirst Then
      bFirst = False
      dWidth = Abs(iLine.CellsU("BeginX") - iLine.CellsU("EndX")) + dIncrement
      If dWidth < 0 Then _
        Exit Function
      dBeginX = iLine.CellsU("BeginX")
    End If
    
    iLine.CellsU("BeginX") = dBeginX
    iLine.CellsU("EndX") = dBeginX + dWidth
  Next iLine
  
  IncrementWidth = True
End Function

' =============
Private Function ScanShapes()
  Set lines_ = New Collection
  Dim iLines As Collection: Set iLines = GetGridLines
  If iLines.Count < 2 Then _
    Exit Function
  
  Dim iShape As Visio.Shape
  Do While iLines.Count <> 0
    Dim curMax As Double: curMax = -999999
    Dim nMax&: nMax = -1
    Dim nItem&: nItem = 1
    For Each iShape In iLines
      If curMax < iShape.CellsU("PinY") Then
        nMax = nItem
        curMax = iShape.CellsU("PinY")
      End If
      nItem = nItem + 1
    Next iShape
    
    Call lines_.Add(iLines.Item(nMax))
    Call iLines.Remove(nMax)
  Loop
  
  rankHeight_ = Abs(lines_.Item(lines_.Count).CellsU("PinY") - lines_.Item(lines_.Count - 1).CellsU("PinY"))
End Function

Private Function GetGridLines() As Collection
  Dim iLines As New Collection
  Dim iShape As Visio.Shape
  For Each iShape In page_.Shapes
    If iShape.Name Like MASK_LEVEL Then _
      Call iLines.Add(iShape)
  Next iShape
  Set GetGridLines = iLines
End Function

Private Function GetNextLineName() As String
  Dim nLine%: nLine = lines_.Count
  Dim sName$
  Do While True
    sName = PREFIX_GRID & nLine
    If Not VsoShapeExists(sName, page_) Then
      GetNextLineName = sName
      Exit Function
    End If
    nLine = nLine + 1
  Loop
End Function
